import 'package:camera/camera.dart';
import 'package:flutter/material.dart';
import 'package:get/get.dart';
import 'package:myhr_facescan/camera_loading.dart';
import 'package:myhr_facescan/match_template/controller.dart';
import 'package:myhr_facescan/models/face_rect.dart';
import 'package:myhr_facescan/models/face_template.dart';
import 'package:myhr_facescan/models/template.dart';
import 'package:myhr_facescan/rectangle_paint.dart';

class MatchTemplateScanner extends StatelessWidget {
  const MatchTemplateScanner({
    super.key,
    required this.licenseKey,
    required this.onFinish,
    this.imageStreamMilliSecond = 100,
    required this.templates,
  });

  final String licenseKey;
  final void Function(bool, TemplateModel?) onFinish;
  final int imageStreamMilliSecond;
  final List<FaceTemplate> templates;

  @override
  Widget build(BuildContext context) {
    var con = Get.put(MatchTemplateController(
        licenseKey: licenseKey,
        onFinish: onFinish,
        imageStreamMilliSecond: imageStreamMilliSecond,
        templates: templates));

    return Obx(() => con.isCameraReady.value
        ? Transform.scale(
            scale: con.previewScale,
            child: CameraPreview(
              con.camController!,
              child: Stack(
                children: [
                  CustomPaint(
                    size: Size(con.camController!.value.previewSize!.height,
                        con.camController!.value.previewSize!.width),
                    painter: RectanglePainter(
                        faces: con.faceMatchs
                            .map((d) => FaceRect(
                                pos: d.position,
                                message: d.id,
                                rectColor:
                                    d.isMatch ? Colors.white : Colors.blue,
                                rectStrokeWidth: d.isMatch ? 2 : 1))
                            .toList(),
                        paperWidth:
                            con.camController!.value.previewSize!.height,
                        paperHeight:
                            con.camController!.value.previewSize!.width),
                  ),
                  Positioned.fill(
                      bottom: 18,
                      left: 36,
                      child: Row(
                        crossAxisAlignment: CrossAxisAlignment.end,
                        children: [
                          IconButton(
                              iconSize: 48,
                              color: Colors.white,
                              onPressed: con.switchCamera,
                              icon: const Icon(Icons.switch_camera_rounded))
                        ],
                      ))
                ],
              ),
            ),
          )
        : const CameraLoading());
  }
}
