import 'dart:ui';

import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'package:myhr_facescan/models/face_rect.dart';

class RectanglePainter extends CustomPainter {
  final List<FaceRect> faces;

  final double paperWidth;
  final double paperHeight;

  RectanglePainter(
      {super.repaint,
      required this.faces,
      required this.paperWidth,
      required this.paperHeight});

  @override
  void paint(Canvas canvas, Size size) {
    const textStyle = TextStyle(
        color: Colors.black,
        // fontSize: 30,
        backgroundColor: Colors.white);

    for (var face in faces) {
      final xc = lerpDouble(0, size.width, face.pos.x / paperWidth)!;
      final yc = lerpDouble(0, size.height, face.pos.y / paperHeight)!;
      final width = lerpDouble(0, size.width, face.pos.width / paperWidth)!;

      var paint = Paint()
        ..color = face.rectColor
        ..style = PaintingStyle.stroke
        ..strokeWidth = face.rectStrokeWidth;

      canvas.drawRect(
          Rect.fromCenter(center: Offset(xc, yc), width: width, height: width),
          paint);

      var textSpan = TextSpan(
        text: face.message,
        style: textStyle,
      );
      final textPainter = TextPainter(
        text: textSpan,
        textDirection: TextDirection.ltr,
      );
      textPainter.layout(
        minWidth: width,
        maxWidth: size.width,
      );
      final xCenter = xc - (width / 2);
      final yCenter = yc - (width / 2) - 20;
      textPainter.paint(canvas, Offset(xCenter, yCenter));
    }
  }

  @override
  bool shouldRepaint(CustomPainter oldDelegate) => true;

  @override
  bool? hitTest(Offset position) {
    print('positionss ${position.dx},${position.dy}');

    for (var face in faces) {}

    return super.hitTest(position);
  }
}
